@@ -5,6 +5,7 @@ import sys
 import pkgutil
 from selenium.webdriver import Firefox
 from selenium.webdriver.common.by import By
+from selenium.webdriver.firefox.webdriver import WebDriver
 from selenium.webdriver.support.ui import WebDriverWait
 from selenium.webdriver.support import expected_conditions as EC
 from selenium.webdriver.firefox.options import Options
@@ -15,12 +16,15 @@ from selenium.common.exceptions import (
     ElementClickInterceptedException,
 )
 from pyvirtualdisplay import Display
+from threading import Event
+from queue import Queue
 from talked import config
+from talked.ffmpeg import assemble_command
 
 msg_queue = None
 
 
-def start(token, queue, recording):
+def start(token: str, queue: Queue, recording: Event, audio_only: bool) -> None:
     global msg_queue
     msg_queue = queue
 
@@ -36,63 +40,27 @@ def start(token, queue, recording):
         backend="xvfb",
         size=(config["video_width"], config["video_height"]),
         color_depth=config["color_depth"],
-    ) as display:
+    ):
         logging.info("Starting browser")
         logging.info(call_link)
         browser = launch_browser(call_link)
         logging.info("Starting ffmpeg process")
-        ffmpeg = subprocess.Popen(
-            [
-                "ffmpeg",
-                "-nostdin",
-                "-nostats",
-                "-hide_banner",
-                "-loglevel",
-                "warning",
-                "-fflags",
-                "+igndts",
-                "-f",
-                "x11grab",
-                "-video_size",
-                f"{config['video_width']}x{config['video_height']}",
-                "-framerate",
-                str(config["framerate"]),
-                "-draw_mouse",
-                "0",
-                "-thread_queue_size",
-                str(config["video_thread_queue_size"]),
-                "-i",
-                display.env()["DISPLAY"],
-                "-f",
-                "pulse",
-                "-ac",
-                "2",
-                "-channel_layout",
-                "stereo",
-                "-thread_queue_size",
-                str(config["audio_thread_queue_size"]),
-                "-i",
-                "0",
-                "-pix_fmt",
-                "yuv420p",
-                "-c:a",
-                config["audio_codec"],
-                "-b:a",
-                config["audio_bitrate"],
-                "-c:v",
-                config["video_codec"],
-                "-crf",
-                str(config["crf"]),
-                "-preset",
-                config["encoding_preset"],
-                "-threads",
-                str(config["encoding_threads"]),
-                (
-                    f"{config['recording_dir']}/"
-                    f"{time.strftime('%Y%m%dT%H%M%S')}_output.mp4"
-                ),
-            ]
-        )
+
+        try:
+            ffmpeg_command = assemble_command(audio_only)
+        except RuntimeError:
+            msg_queue.put(
+                {
+                    "status": "error",
+                    "message": (
+                        "There was an issue with the recording configuration, "
+                        "please contact an administrator."
+                    ),
+                }
+            )
+            graceful_shutdown(browser)
+
+        ffmpeg = subprocess.Popen(ffmpeg_command)
         logging.info("Recording has started")
 
         msg_queue.put(
@@ -116,15 +84,15 @@ def start(token, queue, recording):
         )
 
 
-def launch_browser(call_link):
+def launch_browser(call_link: str) -> WebDriver:
     logging.info("Configuring browser options")
     options = Options()
     options.set_preference("media.navigator.permission.disabled", True)
     options.set_preference("privacy.webrtc.legacyGlobalIndicator", False)
     options.set_preference("full-screen-api.warning.timeout", 0)
     options.add_argument("--kiosk")
-    options.add_argument("--width=1920")
-    options.add_argument("--height=1080")
+    options.add_argument(f"--width={config['video_width']}")
+    options.add_argument(f"--height={config['video_height']}")
 
     logging.info("Creating browser")
     driver = Firefox(options=options)
@@ -148,7 +116,9 @@ def launch_browser(call_link):
     # Press m to mute the microphone, if there is one attached.
     page.send_keys("m")
 
-    switch_to_speaker_view(driver)
+    # If grid view is set to False, switch to speaker view.
+    if not config["grid_view"]:
+        switch_to_speaker_view(driver)
 
     close_sidebar(driver)
 
@@ -163,11 +133,11 @@ def launch_browser(call_link):
     return driver
 
 
-def assemble_call_link(base_url, token):
+def assemble_call_link(base_url: str, token: str) -> str:
     return base_url + "/index.php/call/" + token
 
 
-def is_valid_talk_room(driver):
+def is_valid_talk_room(driver: WebDriver) -> None:
     """Checks if the loaded page is a valid Talk room.
     It looks for the start call / join call button, if it isn't there
     it throws a TimeoutException notifies the HTTP api
@@ -193,7 +163,7 @@ def is_valid_talk_room(driver):
         graceful_shutdown(driver)
 
 
-def change_name_of_user(driver):
+def change_name_of_user(driver: WebDriver) -> None:
     edit_name = WebDriverWait(driver, 10).until(
         EC.visibility_of_element_located(
             (By.CSS_SELECTOR, ".username-form button.icon-rename")
@@ -205,7 +175,7 @@ def change_name_of_user(driver):
     )
 
 
-def join_call(driver):
+def join_call(driver: WebDriver) -> None:
     # Wait for the green Join Call button to appear then click it
     logging.info("Waiting for join call button to appear")
     try:
@@ -244,7 +214,7 @@ def join_call(driver):
         graceful_shutdown(driver)
 
 
-def switch_to_speaker_view(driver):
+def switch_to_speaker_view(driver: WebDriver) -> None:
     # Switch to speaker view
     logging.info("Switching to speaker view")
     try:
@@ -260,7 +230,7 @@ def switch_to_speaker_view(driver):
         )
 
 
-def close_sidebar(driver):
+def close_sidebar(driver: WebDriver) -> None:
     # Close the sidebar
     logging.info("Closing sidebar")
     try:
@@ -281,7 +251,7 @@ def close_sidebar(driver):
     )
 
 
-def close_toasts(driver):
+def close_toasts(driver: WebDriver) -> None:
     while True:
         logging.info("Closing toast")
         try:
@@ -291,12 +261,12 @@ def close_toasts(driver):
             break
 
 
-def load_custom_css(driver):
+def load_custom_css(driver: WebDriver) -> None:
     javascript = pkgutil.get_data("talked", "static/custom_css.js").decode("UTF-8")
     driver.execute_script(javascript)
 
 
-def graceful_shutdown(driver):
+def graceful_shutdown(driver: WebDriver) -> None:
     logging.info("Shutting down...")
     driver.close()
     sys.exit()
